/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2005 John Fitzgibbons and others
Copyright (C) 2007-2008 Kristian Duske

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <stdio.h>
#include "quakedef.h"

// mh - 256 MB heapsize because there's all the more of us to love!
// a lot of stuff is going to go to proper dynamic allocation (the cache and zone for example) and this can go back down
#define DEFAULT_MEMORY 0x10000000

int main (int argc, char *argv[])
{
	SDL_Event	sdlevent;
	quakeparms_t	parms;
	int		t;
	int		done = 0;
	unsigned	time, oldtime, newtime;

#ifdef RMQMSVC
#ifdef _DEBUG
	// this is just less annoying for debugging
	static char argv0[128];
	FILE *f = NULL;

	Q_strcpy (argv0, argv[0]);

	for (t = strlen (argv0); t; t--)
	{
		if (argv0[t] == '/' || argv0[t] == '\\')
		{
			argv0[t] = 0;
			break;
		}
	}

	// make sure it's a full install
	if (f = fopen (va ("%s\\id1\\pak1.pak", argv0), "rb"))
	{
		parms.basedir = argv0;
		fclose (f);
	}
	else parms.basedir = ".";
#else
	parms.basedir = ".";
#endif
#else
	parms.basedir = ".";
#endif

	parms.argc = argc;
	parms.argv = argv;

	COM_InitArgv (parms.argc, parms.argv);

	isDedicated = (COM_CheckParm ("-dedicated") != 0);

	// default memory size
	parms.memsize = DEFAULT_MEMORY;

	if (COM_CheckParm ("-heapsize"))
	{
		t = COM_CheckParm ("-heapsize") + 1;

		if (t < com_argc)
			parms.memsize = Q_atoi (com_argv[t]) * 1024;
	}

	// TODO: dedicated server setup

	parms.membase = malloc (parms.memsize);

	if (!parms.membase)
		Sys_Error ("Not enough memory free; check disk space\n");

	Con_Printf ("FitzQuake %1.2f (c) John Fitzgibbons\n", FITZQUAKE_VERSION);
	Con_Printf ("SDL port (c) Sleepwalkr, Baker\n");
	Con_Printf ("QuakeSpasm %1.2f.%d (c) Ozkan Sezer, Stevenaaus\n", FITZQUAKE_VERSION, QUAKESPASM_VER_PATCH);
	Con_Printf ("RMQ Engine %1.2f.%d (c) Team RMQ\n", FITZQUAKE_VERSION, QUAKESPASM_VER_PATCH);

	Con_Printf ("Host_Init\n");
	Host_Init (&parms);

	oldtime = Sys_FloatTime();

	while (!done)
	{
		// TODO: dedicated server loop

		while (!done && SDL_PollEvent (&sdlevent))
		{
			switch (sdlevent.type)
			{
			case SDL_ACTIVEEVENT:
				if (sdlevent.active.state & (SDL_APPACTIVE | SDL_APPINPUTFOCUS))
				{
					if (sdlevent.active.gain)
					{
						// restore the renderer
						block_drawing = false;
						S_UnblockSound();
					}
					else
					{
						// this prevents typing lag in IM clients when the engine is minimized
						block_drawing = true;
						S_BlockSound();
					}
				}

				break;
			case SDL_MOUSEMOTION:
				IN_MouseMove (sdlevent.motion.xrel, sdlevent.motion.yrel);
				break;
			case SDL_MOUSEBUTTONDOWN:
			case SDL_MOUSEBUTTONUP:

				switch (sdlevent.button.button)
				{
				case SDL_BUTTON_LEFT:
					Key_Event (K_MOUSE1, sdlevent.button.type == SDL_MOUSEBUTTONDOWN);
					break;
				case SDL_BUTTON_RIGHT:
					Key_Event (K_MOUSE2, sdlevent.button.type == SDL_MOUSEBUTTONDOWN);
					break;
				case SDL_BUTTON_MIDDLE:
					Key_Event (K_MOUSE3, sdlevent.button.type == SDL_MOUSEBUTTONDOWN);
					break;
				case SDL_BUTTON_WHEELUP:
					Key_Event (K_MWHEELUP, sdlevent.button.type == SDL_MOUSEBUTTONDOWN);
					break;
				case SDL_BUTTON_WHEELDOWN:
					Key_Event (K_MWHEELDOWN, sdlevent.button.type == SDL_MOUSEBUTTONDOWN);
					break;
				}

				break;
			case SDL_KEYDOWN:
			case SDL_KEYUP:

				// SHIFT + ESC and circomflex always opens the console no matter what
				if ((sdlevent.key.keysym.sym == SDLK_ESCAPE && (sdlevent.key.keysym.mod & KMOD_SHIFT))
						|| (sdlevent.key.keysym.sym == SDLK_CARET))
				{
					if (sdlevent.key.type == SDL_KEYDOWN)
						Con_ToggleConsole_f();
				}
				else
				{
					Key_Event (Key_Map (&(sdlevent.key)), sdlevent.key.type == SDL_KEYDOWN);
				}

				break;
			case SDL_QUIT:
				done = 1;
				break;
			default:
				break;
			}
		}

		newtime = SDL_GetTicks ();
		time = newtime - oldtime;

//		// don't bother with frames that are too short
		// (note - this slows down overall perf)
//		if (time < 0.001) continue;

		Host_Frame (time);

		// throttle the game loop just a little bit - noone needs more than 1000fps, I think
		// mh - this was leading to inconsistent framerates and all manner of evil buggery owing to
		// bad resolution of SDL timers.
//		if (newtime - oldtime < 1)
//			SDL_Delay (1);

		oldtime = newtime;
	}

	Sys_Quit();
	return 0;
}

